function order = compareCones (mx, my)
%% Compare 2x2 Matrices based on Loewner order
%
% Returns 1 if mx > my, -1 if mx < my and 0 if mx == my
% Computation is based on Loewner order.
%
% order = compareCones (mx, my)
%
% Mandatory Parameters:
%
% mx: 2 x 2 symmetric matrix
% my: 2 x 2 symmetric matrix
%
% Optional Parameters:
%
% -
%
% Output Parameters:
%
% order : 1, 0 or -1 depending on order of matrices
%
% Example
%
% compareCones(eye(2), zeros(2,2))
%
% returns 1 since all eigenvalues of my-mx are negative (i.e. mx > my).
%
% See also quicksort

% Copyright (c) 2016 Laurent Hoeltgen <hoeltgen@b-tu.de>
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 3 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
% Street, Fifth Floor, Boston, MA 02110-1301, USA.

% Last revision: 2016-05-17 16:00

%% Parse input and output.

narginchk(2, 2);
nargoutchk(0, 1);

parser               = inputParser;
parser.FunctionName  = mfilename;
parser.KeepUnmatched = false;
parser.StructExpand  = false;

% Parse mandatory arguments

parser.addRequired('mx', ...
    @(x) validateattributes(x, ...
    {'numeric'}, ...
    {'size', [2, 2], 'nonempty', 'nonsparse'}, ...
    mfilename, 'mx', 1));

parser.addRequired('my', ...
    @(x) validateattributes(x, ...
    {'numeric'}, ...
    {'size', [2, 2], 'nonempty', 'nonsparse'}, ...
    mfilename, 'my', 2));

parser.parse(mx, my);

tmp = abs(mx-mx')<100*eps(1);
assert(all(tmp(:)), 'First input matrix is not symetric');
tmp = abs(my-my')<100*eps(1);
assert(all(tmp(:)), 'Second input matrix is not symetric');

%% Run Code

vals = eig(mx - my);
if all(vals > 0)
    %% All eigenvalues of my - mx are nonnegative, i.e. my >= mx.
    order = 1;
elseif all(vals < 0)
    %% All eigenvalues of my - mx are nonpositive, i.e. my <= mx.
    order = -1;
else
    %% We have one positive and one negative eigenvalue. In this case,
    %  we consider the closest matrix (euclidean distance) that is
    %  either positive definite or negative definite.
    
    if (abs(diff(abs(vals))) > 100*eps(1))
        order = sign(vals(abs(vals)==max(abs(vals))));
    else
        %% Both eigenvalues have same magnitude but different signs.
        order = 0;
    end
end
end