function rgb = mat2rgb( mat )
%% Converts symmetric matrix field to rgb colour data
%
% Inverse transform of rgb2mat.
%
% rgb = mat2rgb( mat )
%
% Mandatory Parameters:
%
% mat: nr x nc x 2 x 2 data array  
%
% Optional Parameters:
%
% -
%
% Output Parameters:
%
% rgb : nr x nc x 3 array of colour values
%
% Example
%
% ---
%
% See also rgb2mat

% Copyright (c) 2016 Laurent Hoeltgen <hoeltgen@b-tu.de>
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 3 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
% Street, Fifth Floor, Boston, MA 02110-1301, USA.

% Last revision: 2016-05-13 13:00

%% Parse input and output.

narginchk(1, 1);
nargoutchk(0, 1);

parser               = inputParser;
parser.FunctionName  = mfilename;
parser.KeepUnmatched = false;
parser.StructExpand  = false;

% Parse mandatory arguments

parser.addRequired('mat', ...
    @(x) validateattributes(x, ...
    {'numeric'}, ...
    {'size', [NaN, NaN, 2, 2], 'nonempty', 'nonsparse'}, ...
    mfilename, 'mat', 1));

parser.parse(mat);

%% Run Code

p = zeros(size(mat,1), size(mat,2), 3);

p(:, :, 1) = sqrt(2) * mat(:, :, 1, 2);
p(:, :, 2) = (mat(:, :, 2, 2) - mat(:, :, 1, 1)) / sqrt(2);
p(:, :, 3) = (mat(:, :, 2, 2) + mat(:, :, 1, 1)) / sqrt(2);

C = hypot(p(:, :, 1), p(:, :, 2));
at = atan2(p(:, :, 2), p(:, :, 1));
at(at < 0) = at(at < 0) + 2*pi;
H = at / (2*pi);
Lm = p(:, :, 3);

Ld = (Lm + 1)/2;

mask = C >= 1e-4;
Sd = zeros(size(C));
Sd(mask) = C(mask) ./ (1 - abs(2* Ld(mask) - 1));

[Rm, Gm, Bm] = arrayfun( @hsl2rgb, H, Sd, Ld );

R = round(Rm * 255);
G = round(Gm * 255);
B = round(Bm * 255);

rgb = zeros(size(mat, 1), size(mat, 2), 3);

rgb(:, :, 1) = R;
rgb(:, :, 2) = G;
rgb(:, :, 3) = B;
end

function [r, g, b] = hsl2rgb( H, S, L )
lowLidx = L < (1 / 2);
q = (L .* (1 + S) ) .* lowLidx + (L + S - (L .* S)) .* (~lowLidx);
p = 2 * L - q;
hk = H;

t = zeros( [length(H), 3] );
t(:, 1) = hk + 1 / 3;
t(:, 2) = hk;
t(:, 3) = hk - 1 / 3;

underidx = t < 0;
overidx = t > 1;
t = t + underidx - overidx;

range1 = t < (1 / 6);
range2 = (t >= (1/6) & t < (1/2));
range3 = (t >= (1/2) & t < (2/3));
range4 = t >= (2/3);

P = repmat(p, [1, 3]);
Q = repmat(q, [1, 3]);
rgb_c = (P + ((Q - P) .* 6 .* t)) .* range1 + ...
    Q .* range2 + ...
    (P + ((Q - P) .* 6 .* (2/3 - t))) .* range3 + ...
    P .* range4;

r = rgb_c(1);
g = rgb_c(2);
b = rgb_c(3);
end

