function index = quicksort(input, inl_fcn)
%% Sort input data
%
% Sorts arbitrary data with a quicksort algorithm. Sorting is done with
% help of a provided comparison function.
%
% input may be a multidimensional matrix or cell array. The sort is performed
% along the first non-singleton dimension of Input. The trivial case of a 1x1
% input will not cause a crash.
%
% inl_fcn may be an inline function or function handle. inl_fcn must take as
% arguments the input array and two indices. It then returns 1 if the element(s)
% associated with the first index rank higher than that (those) of the second
% index. It returns -1 if the same ranks lower and 0 if they are considered
% equal.
%
% The order of equal elements is undefined.
%
% index = quicksort(input, inl_fcn)
%
%
% Mandatory Parameters:
%
% input:   arbitrary array or cell array. The first index is the running index
%          labeling the data.
% inl_fcn: inline function or handle to compare data
%
% Optional Parameters:
%
% -
%
% Output Parameters:
%
% index: 1xN index vector which can be used to sort the input argument. Thus,
%        input(index) returns a sorted array.
%
% Example:
%
% Sort a two-column matrix of integers by pairs with the first column in
% ascending order and the second column in descending order when the first
% column elements are equal.
%
%  % matrix to be sorted
%  x = floor(rand(10000,2)*10);
%
%  % inline comparison function
%  i_cmp = inline( ...
%    'sign(sign(m(x,1)-m(y,1))*10-sign(m(x,2)-m(y,2)))', ...
%    'm','x','y');
%
%  % calculate the sorted index
%  order = quicksort(x, i_cmp);
%
%  % obtain the sorted matrix
%  x_sorted = x(order,:);
%
% See also: compareCones

% Copyright (c) 2016 Laurent Hoeltgen <hoeltgen@b-tu.de>
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 3 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
% Street, Fifth Floor, Boston, MA 02110-1301, USA.

% Last revision: 2016-05-17 16:00

sz = size(input);
while (sz(1) == 1) && (length(sz) > 1)
    sz = sz(2:end);
end

index = cumsum(ones(1,sz(1)));

if isa(inl_fcn, 'function_handle')
    index = quicksort_handle(input, inl_fcn, index, 1, sz(1));
else
    index = quicksort_inline(input, inl_fcn, index, 1, sz(1));
end
end

%
% quicksort with an inline
%
function indx = quicksort_inline(inpt, cmp, indx, l, r)
% l and r remain unchanged. they are the left and right
%  bounds of this sorting level
i = l + 1; % leftmost unknown
j = r; % rightmost unknown
p = l; % rightmost equal

if l >= r
    return
end

% this while loop only runs while p == i - 1
while i <= j
    switch cmp(inpt, indx(i), indx(l))
        case 1
            tmp = indx(j);
            indx(j) = indx(i);
            indx(i) = tmp;
            j = j - 1;
        case -1
            i = i + 1;
            break
        otherwise
            p = p + 1;
            i = i + 1;
    end
end

% this is actually the main while loop
%  in this loop i > p + 1
while i <= j
    switch cmp(inpt, indx(i), indx(l))
        case 1
            tmp = indx(j);
            indx(j) = indx(i);
            indx(i) = tmp;
            j = j - 1;
        case -1
            i = i + 1;
        otherwise
            p = p + 1;
            tmp = indx(p);
            indx(p) = indx(i);
            indx(i) = tmp;
            i = i + 1;
    end
end

% swap "less thans" with "equals"
indx(l:j) = [ indx((p + 1):j) indx(l:p) ];

indx = quicksort_inline(inpt, cmp, indx, l, l + j - p);
indx = quicksort_inline(inpt, cmp, indx, i, r);
end

%
% quicksort with a function handle
%
function indx = quicksort_handle(inpt, cmp, indx, l, r)
% l and r remain unchanged. they are the left and right
%  bounds of this sorting level
i = l + 1; % leftmost unknown
j = r; % rightmost unknown
p = l; % rightmost equal

if l >= r
    return
end

% this while loop only runs while p == i - 1
while i <= j
    switch feval(cmp, inpt, indx(i), indx(l))
        case 1
            tmp = indx(j);
            indx(j) = indx(i);
            indx(i) = tmp;
            j = j - 1;
        case -1
            i = i + 1;
            break
        otherwise
            p = p + 1;
            i = i + 1;
    end
end

% this is actually the main while loop
%  in this loop i > p + 1
while i <= j
    switch feval(cmp, inpt, indx(i), indx(l))
        case 1
            tmp = indx(j);
            indx(j) = indx(i);
            indx(i) = tmp;
            j = j - 1;
        case -1
            i = i + 1;
        otherwise
            p = p + 1;
            tmp = indx(p);
            indx(p) = indx(i);
            indx(i) = tmp;
            i = i + 1;
    end
end

% swap "less thans" with "equals"
indx(l:j) = [ indx((p + 1):j) indx(l:p) ];

indx = quicksort_handle(inpt, cmp, indx, l, l + j - p);
indx = quicksort_handle(inpt, cmp, indx, i, r);
end
