function [ M ] = rgb2mat( rgb )
%% Converts rgb colour data to symmetric matrix field
%
% Inverse transform of mat2rgb.
%
% mat = rgb2mat
%
% Mandatory Parameters:
%
% rgb : nr x nc x 3 array of colour values
%
% Optional Parameters:
%
% -
%
% Output Parameters:
%
% mat: nr x nc x 2 x 2 data array (of symmetric matrices)
%
% Example
%
% X = zeros(1, 1, 3);
% X(1, 1, :) = [255, 0, 0];
% rgb2mat(X)
%
% returns sqrt(2)/2 * [0, 1; 1, 0]
%
% See also mat2rgb

% Copyright (c) 2016 Laurent Hoeltgen <hoeltgen@b-tu.de>
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 3 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
% Street, Fifth Floor, Boston, MA 02110-1301, USA.

% Last revision: 2016-05-13 13:30

%% Parse input and output.

narginchk(1, 1);
nargoutchk(0, 1);

parser               = inputParser;
parser.FunctionName  = mfilename;
parser.KeepUnmatched = false;
parser.StructExpand  = false;

% Parse mandatory arguments

parser.addRequired('rgb', ...
    @(x) validateattributes(x, ...
    {'numeric'}, ...
    {'size', [NaN, NaN, 3], 'nonempty', 'nonsparse'}, ...
    mfilename, 'rgb', 1));

parser.parse(rgb);

%% Run Code
M = nan(size(rgb,1), size(rgb,2), 2, 2);

pixel = rgb/255;

[Hd, C, Lm] = arrayfun( @rgb2hclm, pixel(:,:,1), pixel(:,:,2), pixel(:,:,3) );

M(:, :, 1, 1) = (Lm - C .* sin(2 * pi * Hd))/sqrt(2);
M(:, :, 2, 1) = (C .* cos(2 * pi * Hd))/sqrt(2);
M(:, :, 1, 2) = M(:, :, 2, 1);
M(:, :, 2, 2) = (Lm + C .* sin(2 * pi * Hd))/sqrt(2);
end

function [ Hd, chroma, Lm ] = rgb2hclm( Rd, Gd, Bd )
hsl = rgb2hsl( [Rd, Gd, Bd] );
Hd = hsl(1);
Ld = hsl(3);
chroma = max( [Rd, Gd, Bd] ) - min( [Rd, Gd, Bd] );
Lm = 2 * Ld - 1;
end

function hsl = rgb2hsl( rgb )
mx = max( rgb(:) );	% max of the 3 colors
mn = min( rgb(:) );	% min of the 3 colors

hsl = [0, 0, 0];

hsl(3) = (mx + mn) / 2;	% luminance is half of max value + min value

if (mx - mn) == 0,	% if all three colors have same value,
    hsl(2) = 0;	    % then s=0 and
    hsl(1) = 0;     %h is undefined but for practical reasons 0
    return
end

if hsl(3) < 0.5,
    hsl(2) = (mx - mn) / (mx + mn);
else
    hsl(2) = (mx - mn) / (2 - (mx + mn));
end

% if two colors have same value and be the maximum, use the first color
imx = find( rgb(:) == mx); 	% which color has the max
switch(imx(1))
    case 1 %Red is the max color
        hsl(1) = ((rgb(2) - rgb(3)) / (mx - mn)) / 6;
    case 2 %Green is the max color
        hsl(1)=(2+(rgb(3)-rgb(1))/(mx-mn))/6;
    case 3 %Blue is the max color
        hsl(1)=(4+(rgb(1)-rgb(2))/(mx-mn))/6;
end

%if hue is negative, add 1 to get it within 0 and 1
if hsl(1) < 0,
    hsl(1) = hsl(1) + 1;
end

end