function [n, srtCones] = topologicalConeSort(arrCones)
%% Topological sorting inside a 1D array of cones.
%
% Returns sorting array as well as sorted version of input array.
% Computation is based on compareCones and uses a topological ordering to
% overcome input that cannot be compared. Sorting is done in non-decreasing
% order.
%
% [n, srtCones] = topologicalConeSort(arrCones)
%
% Mandatory Parameters:
%
% arrCones: 1D array of 2 x 2 symmetric matrices
%
% Optional Parameters:
%
% -
%
% Output Parameters:
%
% n : sorted entries
% srtCones : sorted matrices
%
% Example
%
% compareCones(eye(2), zeros(2,2))
%
% returns 1 since all eigenvalues of my-mx are negative (i.e. mx > my).
%
% See also quicksort, compareCones

% Copyright (c) 2016 Laurent Hoeltgen <hoeltgen@b-tu.de>
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 3 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
% Street, Fifth Floor, Boston, MA 02110-1301, USA.

% Last revision: 2016-05-18 16:30

num = size(arrCones,1);
G = sparse(num, num);
for i = 1:num
    for j = (i+1):num
        mx = reshape(arrCones(i, :, :), 2, 2);
        my = reshape(arrCones(j, :, :), 2, 2);
        dist = compareCones(mx, my);
        if dist == -1
            %% mx is a predecessor of my
            G(i,j) = norm(eig(my - mx));
        elseif dist == 1
            %% my is a predecessor of mx
            G(j,i) = norm(eig(mx - my));
        end
    end
end
grph = digraph(G,'OmitSelfLoops');
n = toposort(grph, 'Order', 'stable');
srtCones = arrCones(n, :, :);
end