function u = PdeMorphCone(u0, tstep, siz, numit, sigma, mode)
%% PDE based morphology with Rouy Tourin discretisation and explicit scheme
%
% Solves:
%
% u_t = sign(u - u0) ||nabla u||
%
% in the cone colour space with an explicit scheme based on a Rouy-Tourin
% discretisation
%
% Mandatory Parameters:
%
% u0:    input RGB image (bytewise coded)
% tstep: time step size (positive scalar)
% siz:   grid size (positive scalar)
% numit: number of iterations (positive integer)
% sigma: variance of the gaussian pre smoothing applied to u0 for the
%        initialisation (positive scalar)
% mode:  switch to specify the computation and handling of the sign
%        function and gradient magnitude. (integer between 1 and 4)
%        1 : sign = sign of the sum of the eigenvalues
%        2 : use compare cones function which is based on Loewner order
%        3 : apply sign onto eigenvalues and use Jordan Product to preserve
%            symmetry.
%        4 : use standard matrix-matrix product and enforce symmetry by
%            averaging off-diagonals.
%
% Optional Parameters:
%
% -
%
% Output Parameters:
%
% u : resulting RGB image (bytewise coded)
%
% Notes:
%
% - The explicit scheme is not unconditionnally stable. The upper limit seems
%   to be around 0.1, however it may also depend on the mode.
% - I am not sure if a regular grid is really a good idea if our pixels are
%   strange cone shaped objects.
% - This method is extremely slow.
%
% See also RTupWind

% Copyright (c) 2016 Laurent Hoeltgen <hoeltgen@b-tu.de>
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 3 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
% Street, Fifth Floor, Boston, MA 02110-1301, USA.

% Last revision: 2016-12-19 21:00

import ColourCone.rgb2mat
import ColourCone.mat2rgb
import PDEMorphology.RTUpWind

%% Parse input and output.

narginchk(6, 6);
nargoutchk(0, 1);

parser               = inputParser;
parser.FunctionName  = mfilename;
parser.KeepUnmatched = false;
parser.StructExpand  = false;

% Parse mandatory arguments

parser.addRequired('u0', ...
    @(x) validateattributes(x, ...
    {'numeric'}, ...
    {'size', [NaN, NaN, 3], 'nonempty', 'nonsparse'}, ...
    mfilename, 'u0', 1));

parser.addRequired('tstep', ...
    @(x) validateattributes(x, ...
    {'double'}, ...
    {'scalar', 'positive'}, ...
    mfilename, 'tstep', 2));

parser.addRequired('siz', ...
    @(x) validateattributes(x, ...
    {'double'}, ...
    {'scalar', 'positive'}, ...
    mfilename, 'siz', 3));

parser.addRequired('numit', ...
    @(x) validateattributes(x, ...
    {'double'}, ...
    {'scalar', 'integer', 'positive'}, ...
    mfilename, 'numit', 4));

parser.addRequired('sigma', ...
    @(x) validateattributes(x, ...
    {'double'}, ...
    {'scalar', 'positive'}, ...
    mfilename, 'sigma', 5));

parser.addRequired('mode', ...
    @(x) validateattributes(x, ...
    {'double'}, ...
    {'scalar', 'integer', 'positive'}, ...
    mfilename, 'mode', 6));

parser.parse(u0, tstep, siz, numit, sigma, mode);

%% Run Code

% Transform to Matrix Cones
uMat = rgb2mat(u0);

% Notes:
% 1) taking u0 as initialisaion yields a fixed-point since sign(u-f) = 0.
% 2) taking any constant function as initialisation yields a fixed point as
%    well, since the gradient is always 0.
% We initialise with a gaussian smoothed variant of u0.
init = nan(size(uMat));
for ii = 1:2
    for jj = 1:2
        init(:,:,ii,jj) = imgaussfilt(uMat(:,:,ii,jj), sigma);
    end
end
% init = rgb2mat(imgaussfilt(u0, sigma));

% Perform iterations of the upwind scheme.
vMat = RTUpWind(init, uMat, tstep, siz, numit, mode);

% Convert back to RGB
u = mat2rgb(vMat);

end





