function nabla = RTGradMag(u, siz)
%% Compute matrix Gradient magnidue of 2x2 matrix valued image with Rouy-Tourin 
%
% Computes derivatives based on a finite difference scheme. Acts channel
% wise
%
% Mandatory Parameters:
%
% u:   input image (colour cone coded)
% siz: grid size (positive scalar)
%
% Optional Parameters:
%
% -
%
% Output Parameters:
%
% nabla : resulting gradient
%
% See also RTdiff, RTGrad

% Copyright (c) 2016 Laurent Hoeltgen <hoeltgen@b-tu.de>
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 3 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
% Street, Fifth Floor, Boston, MA 02110-1301, USA.

% Last revision: 2016-12-15 08:20

import PDEMorphology.RTGrad
import MatrixOps.symmetrise

%% Parse input and output.

narginchk(2, 2);
nargoutchk(0, 1);

parser               = inputParser;
parser.FunctionName  = mfilename;
parser.KeepUnmatched = false;
parser.StructExpand  = false;

% Parse mandatory arguments

parser.addRequired('u', ...
    @(x) validateattributes(x, ...
    {'numeric'}, ...
    {'size', [NaN, NaN, 2, 2], 'nonempty', 'nonsparse'}, ...
    mfilename, 'u', 1));

parser.addRequired('siz', ...
    @(x) validateattributes(x, ...
    {'double'}, ...
    {'scalar', 'positive'}, ...
    mfilename, 'siz', 2));

parser.parse(u, siz);

%% Run Code

[dx, dy] = RTGrad(u, siz);
% Notes: 
% - The regularisation with 10e-8 ensures that the matrix is always positiv
%   definite. Matlab throws a warning for the 0 matrix.
% - sqrtm may return complex numbers with imaginary part 0. abs makes sure that
%   all entries are real
% - sqrtm doesn't return perfectly symmetric matrices, even if its input is. We
%   apply symmetrise to eliminate this issue.
nabla = cellfun(@(x,y) symmetrise(abs(sqrtm(x*x + y*y + 10e-8*eye(2,2) ))), ...
    dx, dy, 'UniformOutput', false);
end
