function v = RTUpWind(u, u0, tstep, siz, its, mode)
%% Explicit Scheme
%
% Solves:
%
% u_t = sign(u - u0) ||nabla u||
%
% in the cone colour space with an explicit scheme based on a Rouy-Tourin
% discretisation
%
% Mandatory Parameters:
%
% u:     initialisation
% u0:    input image (colour cone coded)
% tstep: time step size (positive scalar)
% siz:   grid size (positive scalar)
% its:   number of iterations (positive integer)
% mode:  switch to specify the computation and handling of the sign
%        function and gradient magnitude. (integer between 1 and 4)
%        1 : sign = sign of the sum of the eigenvalues
%        2 : use compare cones function which is based on Loewner order
%        3 : apply sign onto eigenvalues and use Jordan Product to preserve
%            symmetry.
%        4 : use standard matrix-matrix product and enforce symmetry by
%            averaging off-diagonals.
%
% Optional Parameters:
%
% -
%
% Output Parameters:
%
% v : resulting image (colour cone coded)
%
% Notes:
%
% - The explicit scheme is not unconditionnally stable. The upper limit seems
%   to be around 0.1, however it may also depend on the mode.
% - I am not sure if a regular grid is really a good idea if our pixels are
%   strange cone shaped objects.
% - This method is extremely slow.
%
% See also PdeMorhpCone

% Copyright (c) 2016 Laurent Hoeltgen <hoeltgen@b-tu.de>
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 3 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
% Street, Fifth Floor, Boston, MA 02110-1301, USA.

% Last revision: 2016-12-05 16:00

import ColourCone.compareCones
import MatrixOps.sgnMat
import MatrixOps.ssgnMat
import MatrixOps.JordanProduct
import MatrixOps.symmetrise
import PDEMorphology.RTGradMag

%% Parse input and output.

narginchk(6, 6);
nargoutchk(0, 1);

parser               = inputParser;
parser.FunctionName  = mfilename;
parser.KeepUnmatched = false;
parser.StructExpand  = false;

% Parse mandatory arguments

parser.addRequired('u', ...
    @(x) validateattributes(x, ...
    {'numeric'}, ...
    {'size', [NaN, NaN, 2, 2], 'nonempty', 'nonsparse'}, ...
    mfilename, 'u', 1));

parser.addRequired('u0', ...
    @(x) validateattributes(x, ...
    {'numeric'}, ...
    {'size', [NaN, NaN, 2, 2], 'nonempty', 'nonsparse'}, ...
    mfilename, 'u0', 2));

parser.addRequired('tstep', ...
    @(x) validateattributes(x, ...
    {'double'}, ...
    {'scalar', 'positive'}, ...
    mfilename, 'tstep', 3));

parser.addRequired('siz', ...
    @(x) validateattributes(x, ...
    {'double'}, ...
    {'scalar', 'positive'}, ...
    mfilename, 'siz', 4));

parser.addRequired('its', ...
    @(x) validateattributes(x, ...
    {'double'}, ...
    {'scalar', 'integer', 'positive'}, ...
    mfilename, 'its', 5));

parser.addRequired('mode', ...
    @(x) validateattributes(x, ...
    {'double'}, ...
    {'scalar', 'integer', 'positive'}, ...
    mfilename, 'mode', 6));

parser.parse(u, u0, tstep, siz, its, mode);

%% Run Code

i = 0;
while (i < its)
    i = i + 1;
    
    fprintf('Iteration: %g\n', i);
    
    uc = cellfun(@(x) squeeze(x), num2cell(u, [3, 4]), ...
        'UniformOutput', false);
    du = num2cell(u-u0, [3, 4]);
    
    asy = cellfun(@(x)  checkAsymmetry (squeeze(x)), du);
    fprintf('Maximal deviation from symmetry of du: %g\n', max(asy(:)));

    % Compute sgn of u-u0
    % As an alternative to sgnMat one may use compareCones, which returns a
    % scalar, whereas sgnMat returns a 2x2 matrix.

    % Compute gradient magnitude
    gM = RTGradMag(u, siz);
    
    asy = cellfun(@(x)  checkAsymmetry (x), gM);
    fprintf('Maximal deviation from symmetry of gM: %g\n', max(asy(:)));
    
    % perform explicit time step
    % Note: sgn and gM are symmetric matrices, however, their product is
    % not necessarily symmetric. Even if sgn is a diagonal matrix, we
    % cannot ensure symmetry. The alternatives are:
    % 1. enforce symmetry by averaging upper and lower off-diagonals
    % 2. use a scalar valued sign function like ssgnMat or compareCones
    % 3. use the Jordan product.
    switch mode
        case 1
            %% sign of trace of eigenvalues (scalar valued)
            
            sgn = cellfun( @(x) ssgnMat(squeeze(x)), ...
                du, 'UniformOutput', false);
            
            w = cellfun(@(x, y, z) x + tstep*y*z, ...
                uc, sgn, gM, 'UniformOutput', false);
        case 2
            %% compare cones function (scalar valued)
            
            sgn = cellfun( @(x) compareCones(squeeze(x), [0, 0; 0, 0]), ...
                du, 'UniformOutput', false);
            
            w = cellfun(@(x, y, z) x + tstep*y*z, ...
                uc, sgn, gM, 'UniformOutput', false);            
        case 3
            %% Jordan Product
            
            sgn = cellfun( @(x) sgnMat(squeeze(x)), ...
                du, 'UniformOutput', false);
            
            asy = cellfun(@(x)  checkAsymmetry (x), sgn);
            fprintf('Maximal deviation from symmetry of sgn: %g\n', ...
                max(asy(:)));
            
            w = cellfun(@(x, y, z) x + tstep*JordanProduct(y, z), ...
                uc, sgn, gM, 'UniformOutput', false);
        case 4
            %% Enforce symmetry by averaging
            
            sgn = cellfun( @(x) sgnMat(squeeze(x)), ...
                du, 'UniformOutput', false);
            
            asy = cellfun(@(x)  checkAsymmetry (x), sgn);
            fprintf('Maximal deviation from symmetry of sgn: %g\n', ...
                max(asy(:)));
                        
            w = cellfun(@(x, y, z) x + tstep*symmetrise(y*z), ...
                uc, sgn, gM, 'UniformOutput', false);
        otherwise
            error('Unknown mode');
    end
    
    fprintf('Statistics (mean, var, max diff, min diff, avg diff):\n');
    tmp = cellfun(@(x,y) norm(x-y,'fro'), w, uc);
    fprintf('avg dist: %g\n', mean(tmp(:)));
    fprintf('var dist: %g\n', var(tmp(:)));
    fprintf('min dist: %g\n', min(tmp(:)));
    fprintf('max dist: %g\n', max(tmp(:)));
    
    asy = cellfun(@(x)  checkAsymmetry (x), w);
    fprintf('Maximal deviation from symmetry of w: %g\n', max(asy(:)));
    
    u = cell2mat( ...
        cellfun( @(x) reshape(x, [1, 1, 2, 2]), w, ...
        'UniformOutput', false));
end
v = u;
end

function mx = checkAsymmetry (A)
tmp = arrayfun(@abs, A(2,1) - A(1,2));
mx = tmp(:);
end