function z = RTdiff(u, siz, dim)
%% Compute derivative of 2x2 matrix valued image with Rouy-Tourin 
%
% Computes derivative based on a finite difference scheme. Acts channel
% wise
%
% Mandatory Parameters:
%
% u:   input image (colour cone coded)
% siz: grid size (positive scalar)
% dim: dimension along which to compute derivative.
%
% Optional Parameters:
%
% -
%
% Output Parameters:
%
% z : resulting derivative
%
% See also RTGrad, RTGradMag

% Copyright (c) 2016 Laurent Hoeltgen <hoeltgen@b-tu.de>
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 3 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
% Street, Fifth Floor, Boston, MA 02110-1301, USA.

% Last revision: 2016-12-02 10:00

import MatrixOps.maxMat
import MatrixOps.minMat

narginchk(3, 3);
nargoutchk(0, 1);

parser               = inputParser;
parser.FunctionName  = mfilename;
parser.KeepUnmatched = false;
parser.StructExpand  = false;

% Parse mandatory arguments

parser.addRequired('u', ...
    @(x) validateattributes(x, ...
    {'numeric'}, ...
    {'size', [NaN, NaN, 2, 2], 'nonempty', 'nonsparse'}, ...
    mfilename, 'u', 1));

parser.addRequired('siz', ...
    @(x) validateattributes(x, ...
    {'double'}, ...
    {'scalar', 'positive'}, ...
    mfilename, 'siz', 2));

parser.addRequired('dim', ...
    @(x) validateattributes(x, ...
    {'char'}, ...
    {'scalar'}, ...
    mfilename, 'dim', 3));

parser.parse(u, siz, dim);

validatestring(dim, {'x', 'y'});

%% Run Code

dp = fdDiff(u, 1, dim); % Compute forward differences
dm = bdDiff(u, 1, dim); % Compute backward differences

% Reorganize nr x nc x 2 x 2 array into a nr x nc cell array of 2 x 2 matrices
tp = num2cell(dp, [3, 4]); 
tm = num2cell(dm, [3, 4]);

% Note: The formula for the discretisation listed in:
%
% A Directional Rouy-Tourin Scheme for Adaptive Matrix-Valued Morphology
%
% does not correspond to the formula in the original work:
%
% A viscosity solutions approach to shape-from-shading
%
% This is the discretisation from the original Rouy-Tourin paper.

% compute maximum between (-1) backward difference and 0 matrix
t1 = cellfun(@(x) maxMat( squeeze(x), [0, 0; 0, 0])/siz, tm, ...
    'UniformOutput', false);
% compute maximum between forward difference and 0 matrix
t2 = cellfun(@(x) (-1)*minMat( squeeze(x), [0, 0; 0, 0])/siz, tp, ...
    'UniformOutput', false);
% compute maximum between the maxima t1 and t2.
z = cellfun(@(x, y) maxMat(x, y), t1, t2, 'UniformOutput', false);
end
